(* file: babyloniansqrt.ml
 * author: Bob Muller
 *)
let show i lo hi guess error =
  Printf.printf "try=%d: lo=%.10f, hi=%.10f, guess=%.10f, error=%.10f\n" i lo hi guess error

(* sqrt : float -> float -> int -> float option
 *
 * The call (sqrt x epsilon n) will return an (option of) approximation 
 * of the square root of x such that the square of the result is within
 * epsilon of x. The algorithm computes approximations iteratively and 
 * returns None if it cannot compute the approximation within n iterations.
 *)
let sqrt x ?epsilon:(epsilon=1e-6) ?n:(n=50) =
  let rec iterate i lo hi =
    match i > n with
    | true  -> None
    | false ->
       let guess = (lo +. hi) /. 2.0 in
       let guessSquared = guess ** 2.0 in
       let error = abs_float(x -. guessSquared) in
       let _ = show i lo hi guess error 
       in
       (match error < epsilon with
	| true -> Some guess
	| false -> (match guessSquared < x with
		    | true  -> iterate (i + 1) guess hi
		    | false -> iterate (i + 1) lo guess))
  in
  iterate 1 0.0 x
