(* file: checks.ml
 * author: Bob Muller
 * date: Septemeber 17, 2015
 *
 * A few examples using the Universe library with repetition.
 *)

open World
open Color
open Image
open Cs1101

let displayWidth = 800.0
let displayHeight = displayWidth

(* makeCheck : int -> float -> Image.t

 The call (makeCheck n side) returns a square that is black if n is
 even and a random color if n is odd.
 *)
let makeCheck n side =
  match n mod 2 = 0 with
  | true  -> rectangle side side Color.black
  | false -> rectangle side side Color.red

(* addColumnOfChecks : int -> float -> int -> Image.t -> Image.t

 The call (addColumnOfChecks col side n img) returns the image
 obtained from img by attaching a column of alternately colored
 checks in column col. 
*)
let addColumnOfChecks col side n image =
  let x = side *. (i2F (col - 1)) in
  let emptyColumn = Image.empty_scene side displayHeight in
  let rec addChecks row image =
    match row = 0 with
    | true  -> image
    | false -> let y = side *. (i2F (row - 1)) in
	       let check = makeCheck (row + col) side in
	       let newImage = place_image check (0., y) image
	       in
	       addChecks (row - 1) newImage in
  let newColumn = addChecks n emptyColumn
  in
  place_image newColumn (x, 0.) image
	      
(* checks : int -> Image.t

 The call (checks n) returns an n x n checkerboard.
 *)
let checks n =
  let side = displayWidth /. (i2F n) in
  let empty = Image.empty_scene displayWidth displayHeight in
  let rec addColumns col image =
    match col = 0 with
    | true  -> image
    | false -> let newImage = addColumnOfChecks col side n image
	       in
	       addColumns (col - 1) newImage
  in
  addColumns n empty

let stop _ = true     (* draw exactly once *)
let stop _ = false    (* draw ad infinitum *)

(* draw : 'a -> Image.t   this version for demo of addStripe 
 *)
let draw _ = checks 5

let _ = World.big_bang ()
		       ~name:"Stripe Demo"
		       ~width:(f2I displayWidth)
		       ~height:(f2I displayHeight)
		       ~stop_when:stop
		       ~to_draw:draw
		       ~to_draw_last:draw
