(* file: portmanteau.ml
 * author: Bob Muller
 * inspiration: Tom Murphy VII 
 *
 * material from CS1101, Boston College
 *
 * Compute the n-overlap portmanteau of words in a vocabulary file.
 *
 * See: http://www.cs.cmu.edu/~tom7/portmantout/
 * 
 * Compile: > ocamlc -o portmanteau portmanteau.ml
 * Run:     > ./portmanteau vocabShort.txt 3
 *)
let readWords filename =
  let inch = open_in filename in
  let rec repeat words =
  try 
    repeat ((input_line inch)::words)
  with
    End_of_file -> close_in inch;
		   words
  in
  repeat []

let makeOutFileName infile m n =
  let stem = String.sub infile 0 (String.index infile '.')
  in
  stem ^ "." ^ (string_of_int m) ^ "." ^ (string_of_int n) ^ ".txt"

let makeTriple word n =
  let len = String.length word in
  match len >= n with
  | true  -> Some (word, String.sub word 0 n, String.sub word (len - n) n)
  | false -> None

let matcher w s maybe =
  match maybe with
    None -> None
  | Some(w', p, _) -> if s = p then Some (w, w') else None

(* portmanteau : string -> int -> string list
 *
 * The call (portmanteau filename n) constructs a length-two portmanteau
 * of words found in file filename overlapping by n letters.
 *)							
let portmanteau filename n =
  let words = readWords filename in
  let fixen = List.map (fun word -> (makeTriple word n)) words in
  let matches triple =
    match triple with
    | None -> []
    | Some(w, _, s) -> List.map (fun triple -> matcher w s triple) fixen in
  let maybes = List.fold_left (@) [] (List.map matches fixen) in
  let somes = List.filter (fun maybe -> maybe <> None) maybes in
  let attacher front back =
    let fl = String.length front
    in
    (String.sub front 0 (fl - n)) ^ back
  in
  List.iter
    (fun s -> print_string (s ^ ", "))
    (List.map (fun (Some(front, back)) -> attacher front back) somes)

let () =
  let file = Sys.argv.(1) in
  let n = int_of_string Sys.argv.(2)
  in
  portmanteau file n
