(* file: setting.ml
 * author: Bob Muller
 * date: October 8, 2015
 *
 * A simple animation example, a setting sun over water.
 *
 * To run it:
 * > make
 * > ./setting
 *
 *)
open Cs1101

let displayWidth = 600.
let displayHeight = displayWidth

let radius = displayWidth /. 6.0
let dropAmount = 5.0

let sky   = Image.rectangle displayWidth displayHeight Color.lightBlue
let water = Image.rectangle displayWidth (displayHeight /. 2.0) Color.blue
let sun   = Image.circle radius Color.orange

(* Just need to keep track of changing y-component of sun.
*)
type world_t = float
		 
let draw (y : world_t) =
  let x = displayWidth /. 2.0 -. radius in
  let sky = Image.place_image sun (x, y -. radius) sky
  in
  Image.place_image water (0.0, displayHeight -. radius) sky

let drop (y : world_t) = World.World(y +. dropAmount)

let sunHasSet (y : world_t) = y >= displayHeight

let _ = World.big_bang 0.0
		       ~name: "Setting Sun"
		       ~width: (f2I displayWidth)
		       ~height: (f2I displayHeight)
		       ~to_draw: draw
		       ~rate: 0.1
		       ~on_tick: drop
		       ~stop_when: sunHasSet
		       ~to_draw_last: draw
