(* file: stripes.ml
 * author: Bob Muller
 * date: Septemeber 15, 2015
 *
 * A few examples using the Universe library with repetition.
 *)

open World
open Color
open Image
open Cs1101

let displayWidth = 800.0
let displayHeight = displayWidth

(* stripe0 : unit -> Image.t  

 The call (stripe0 ()) returns an image with a randomly colored vertical
 stripe on the right side.
*)
let stripe0 () =
  let empty = Image.empty_scene displayWidth displayHeight in
  let x = displayWidth /. 2.0 in
  let width = displayWidth /. 2.0 in
  let color = Cs1101.randomColor() in
  let stripe = Image.rectangle width displayHeight color
  in
  place_image stripe (x, 0.) empty

(* stripe1 : float -> float -> Image.t  

 The call (stripe1 x width) returns an image with a randomly colored 
 vertical stripe of the given width with upper left corner at the given
 x. NB: stripe1 is obtained from stripe0 by abstracting with respect to 
 both x and width.
*)
let stripe1 x width =
  let empty = Image.empty_scene displayWidth displayHeight in
  let color = Cs1101.randomColor() in
  let stripe = Image.rectangle width displayHeight color
  in
  place_image stripe (x, 0.) empty
  
(* addStripe : float -> float -> Image.t -> Image.t

 The call (addStripe x width img) returns the image obtained from img
 by dropping a randomly colored vertical stripe of the given width
 with upper left corner at the given x. NB: addStripe is obtained from
 stripe1 by abstracting with respect to the image on which the stripe
 is placed.
 *)
let addStripe x width image =
  let color = Cs1101.randomColor() in
  let stripe = rectangle width displayHeight color
  in
  place_image stripe (x, 0.) image

(* stripes : int -> Image.t 

 The call (stripes n) returns an image with n randomly colored vertical
 stripes.
*)
let stripes n =
  let width = displayWidth /. (i2F n) in
  let empty = Image.empty_scene displayWidth displayHeight in
  let rec addStripes m image =
    match m = 0 with
    | true  -> image
    | false -> let x = (i2F (m - 1)) *. width in
	       let newImage = addStripe x width image
	       in
	       addStripes (m - 1) newImage
  in
  addStripes n empty

let stop _ = true     (* draw exactly once *)
let stop _ = false    (* draw ad infinitum *)

(* draw : 'a -> Image.t   this version for demo of addStripe 
 *)
let draw _ = 
  let x = displayWidth /. 4.0 in
  let width = (displayWidth /. 4.0) in
  let empty = Image.empty_scene displayWidth displayHeight in
  let firstImage = addStripe x width empty
  in
  addStripe 0. 20. firstImage

(* draw : 'a -> Image.t   this version for demo of stripes 
 *)
let draw _ = stripes (1 + Random.int 10)
		     
let _ = World.big_bang ()
		       ~name:"Stripe Demo"
		       ~width:(f2I displayWidth)
		       ~height:(f2I displayHeight)
		       ~stop_when:stop
		       ~to_draw:draw
		       ~to_draw_last:draw
